// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include "Atomic.h"
#include "Curry.h"
#include "Muesli.h"
#include "TestAtomic.h"
#include "Utility.h"

// main function

void msl::test::atomic::testAtomic() {
	msl::utl::printv("Welcome to the test suite for the Atomic skeleton.\n");

	if(msl::Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");

		testConstructors();
		testDestructor();
		testFunctions();

		msl::utl::printv("\nall test passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

// argument functions

int* msl::test::atomic::f(int* i) {
	int* result = new int;

	*result = (*i)++;

	return result;
}

// test functions

void msl::test::atomic::testConstructors() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 1. testing constructors              *\n");
	msl::utl::printv("****************************************\n");

	msl::Atomic<int, int> atomic1(f, PROCESSORS);
	msl::Atomic<int, int> atomic2(DFct1<int*, int*>(curry(f)), PROCESSORS);
	
	msl::utl::printv("\nall constructor tests passed!\n");
}

void msl::test::atomic::testDestructor() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 2. testing destructor                *\n");
	msl::utl::printv("****************************************\n");

	msl::Atomic<int, int>* atomic = new Atomic<int, int>(f, PROCESSORS);
	delete atomic;
	
	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::atomic::testFunctions() {
	msl::Atomic<int, int> atomic(f, PROCESSORS);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 3. testing functions                 *\n");
	msl::utl::printv("****************************************\n");

	msl::utl::printv("\ntesting atomic.copy()");
	msl::Atomic<int, int>* copy = (Atomic<int, int>*)atomic.copy();

	msl::utl::printv("\ntesting atomic.show(): ");
	atomic.show();

	msl::utl::printv("testing atomic.start()");
	atomic.start();
	
	msl::utl::printv("\n\nall function tests passed!\n");
}



int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::atomic::testAtomic();
	msl::TerminateSkeletons();
	return 0;
}
