// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include <stdarg.h>
#include <stdio.h>

#include "Curry.h"
#include "DistributedArray.h"
#include "Muesli.h"
#include "TestDistributedArray.h"
#include "Utility.h"

// main function

void msl::test::da::testDistributedArray() {
	msl::utl::printv("Welcome to the test suite for the DistributedArray.\n");

	if(Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");
		msl::utl::printv("The test parameters are as follows:\n");
		msl::utl::printv("size    = %d\n", SIZE);
		msl::utl::printv("initial = %d\n", INITIAL);

		testAuxiliaryFunctions();
		testConstructors();
		testDestructor();
		testSkeletons();

		msl::utl::printv("\nall test passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

// argument functions

int msl::test::da::addIndex(int i, int x, int y) {
	return x + y + i;
}

int msl::test::da::add(int x, int y) {
	return x + y;
}

int msl::test::da::decrease(int x) {
	return x - 1;
}

int msl::test::da::identity(int index) {
	return index;;
}

int msl::test::da::increase(int x) {
	return x + 1;
}

int msl::test::da::increaseIndex(int i, int x) {
	return x + i;
}

int msl::test::da::square(int x) {
	return x * x;
}

int msl::test::da::leftShiftPartition(int index) {
	return (index + PROCESSORS - 1) % PROCESSORS;
}

int msl::test::da::noShift(int index) {
	return index;
}

int msl::test::da::rightShiftElement(int index) {
	return (index + 1) % SIZE;
}

// test functions

void msl::test::da::testAuxiliaryFunctions() {
	msl::DistributedArray<int> da(SIZE, identity);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 1. testing auxiliary functions       #\n");
	msl::utl::printv("########################################\n");
	
	msl::utl::printv("\nda = ");
	da.show();

	msl::utl::printv("\n");
	msl::utl::printv("da.getFirst()     = %d\n", da.getFirst());
	msl::utl::printv("da.getSize()      = %d\n", da.getSize());
	msl::utl::printv("da.getLocalSize() = %d\n", da.getLocalSize());

	for(int i = -1; i < SIZE + 1; i++) {
		msl::utl::printv("da.isLocal(%2d)    = %d\n", i, da.isLocal(i));
	}

	for(int i = 0; i < SIZE / PROCESSORS; i++) {
		da.setLocal(i, (i + 1));

		msl::utl::printv("da.setLocal(%d, %d), da\t= ", i, (i + 1));
		da.show();
		msl::utl::printv("\n");
	}

	for(int i = -1; i < SIZE + 1; i++) {
		da.set(i, (i + 3.0));

		msl::utl::printv("da.set(%2d, %d), da\t= ", i, (i + 3));
		da.show();
		msl::utl::printv("\n");
	}

	for(int i = 0; i < SIZE; i++) {
		msl::utl::printv("da.get(%d) = %d\n", i, da.get(i));
	}

	for(int i = 0; i < SIZE; i++) {
		msl::utl::printv("da[%d] = %d\n", i, da[i]);
	}

	for(int i = 0; i < SIZE / PROCESSORS; i++) {
		msl::utl::printv("da.getLocal(%d) = %d\n", i, da.getLocal(i));
	}

	msl::utl::printv("da.getId() = %d\n", da.getId());
	msl::utl::printv("da.copy()\t\t= ");
	da.copy()->show();
	msl::utl::printv("\n");
	msl::utl::printv("da.copyWithGap()\t= ");
	da.copyWithGap(SIZE * 2, INITIAL)->show();
	msl::utl::printv("\n");
	msl::utl::printv("operator <<, da\t\t= ");
	std::cout << da;

	msl::utl::printv("\n\nall auxiliary function tests passed!\n");
}

void msl::test::da::testConstructors() {
	int* b = new int[SIZE];

	for(int i = 0; i < SIZE; i++) {
		b[i] = i + 1;
	}

	msl::DistributedArray<int> da1(SIZE);
	msl::DistributedArray<int> da2(SIZE, INITIAL);
	msl::DistributedArray<int> da3(SIZE, b);
	msl::DistributedArray<int> da4(SIZE, identity);
	msl::DistributedArray<int> da5(SIZE, curry(identity));
	msl::DistributedArray<int> da6(da5);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 2. testing constructors              #\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("\nb       = [");

	for(int i = 0; i < SIZE; i++) {
		msl::utl::printv("%d", b[i]);

		if(i < SIZE - 1) {
			msl::utl::printv(", ");
		}
		else {
			msl::utl::printv("]\n");
		}
	}

	msl::utl::printv("\nda1, all values uninitialized\n");
	da1.show();
	msl::utl::printv("\nda2, all values set to initial\n");
	da2.show();
	msl::utl::printv("\nda3, values initialized with b\n");
	da3.show();
	msl::utl::printv("\nda4, values initialized with f\n");
	da4.show();
	msl::utl::printv("\nda5, values initialized with curry(f)\n");
	da5.show();
	msl::utl::printv("\nda6, values initialized with da5\n");
	da6.show();
	msl::utl::printv("\nall constructor tests passed!\n");
}

void msl::test::da::testDestructor() {
	msl::DistributedArray<int>* da = new msl::DistributedArray<int>(SIZE);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 3. testing destructor                #\n");
	msl::utl::printv("########################################\n");

	delete da;

	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::da::testSkeletons() {
	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4. testing skeletons                 #\n");
	msl::utl::printv("########################################\n");

	testSkeletonsComputation();
	testSkeletonsCommunication();

	msl::utl::printv("\nall skeleton tests passed!\n");
}

void msl::test::da::testSkeletonsComputation() {
	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4.1 testing computation skeletons    #\n");
	msl::utl::printv("########################################\n");

	testSkeletonsMap();
	testSkeletonsZip();
	//testSkeletonsRemaining(); //produces errors

	msl::utl::printv("\nall computation skeleton tests passed!\n");
}

void msl::test::da::testSkeletonsMap() {
	msl::DistributedArray<int> da(SIZE, identity);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4.1.1 testing map                    #\n");
	msl::utl::printv("########################################\n");

	msl::utl::printv("\nda = ");
	da.show();
	msl::utl::printv("\nmap,             da = ");
	da.copy()->map(increase)->show();
	msl::utl::printv("\nmapIndex,        da = ");
	da.copy()->mapIndex(increaseIndex)->show();
	msl::utl::printv("\nmapInPlace,      da = ");
	da.mapInPlace(increase);
	da.show();
	msl::utl::printv("\nmapIndexInPlace, da = ");
	da.mapIndexInPlace(increaseIndex);
	da.show();
	msl::utl::printv("\nnote: mapPartitionInPlace not tested\n");

	msl::utl::printv("\nall map skeleton tests passed!\n");
}

void msl::test::da::testSkeletonsZip() {
	msl::DistributedArray<int> da1(SIZE, identity);
	msl::DistributedArray<int> da2(SIZE, identity);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4.1.2 testing zip                    #\n");
	msl::utl::printv("########################################\n");

	msl::utl::printv("\nda1 = ");
	da1.show();
	msl::utl::printv("\nda2 = ");
	da2.show();

	msl::utl::printv("\nzip,                 da = ");
	da1.copy()->zipWith(da2, add).show();
	msl::utl::printv("\nzipWithIndex,        da = ");
	da1.copy()->zipWithIndex(da2, addIndex).show();
	msl::utl::printv("\nzipWithInPlace,      da = ");
	da1.zipWithInPlace(da2, add);
	da1.show();
	msl::utl::printv("\nzipWithIndexInPlace, da = ");
	da1.zipWithIndexInPlace(da2, addIndex);
	da1.show();

	msl::utl::printv("\n\nall zip skeleton tests passed!\n");
}

void msl::test::da::testSkeletonsRemaining() {
	msl::DistributedArray<int> da(SIZE, identity);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4.1.3 testing remaining skeletons    #\n");
	msl::utl::printv("########################################\n");

	msl::utl::printv("\nda = ");
	da.show();
	msl::utl::printv("\n");

	msl::utl::printv("fold                                = %d\n", da.fold(add));
	msl::utl::printv("mapIndexInPlaceFold                 = %d\n", da.mapIndexInPlaceFold(increaseIndex, add));
	msl::utl::printv("mapIndexInPlacePermutePartition, da = ");
	da.mapIndexInPlacePermutePartition(increaseIndex, leftShiftPartition);
	da.show();
	msl::utl::printv("\n");
	msl::utl::printv("mapIndexInPlaceScan,             da = ");
	da.mapIndexInPlaceScan(increaseIndex, add);
	da.show();
	msl::utl::printv("\n");
	msl::utl::printv("scan,                            da = ");
	da.scan(add);
	da.show();
	msl::utl::printv("\n\nall remaining skeleton tests passed!\n");
}

void msl::test::da::testSkeletonsCommunication() {
	int* b = new int[SIZE];

	msl::DistributedArray<int> da1(SIZE, identity);
	msl::DistributedArray<int> da2(da1);
	msl::DistributedArray<int> da3(da1);
	msl::DistributedArray<int> da4(da1);
	msl::DistributedArray<int> da5(da1);
	msl::DistributedArray<int> index(SIZE, 0);

	msl::utl::printv("\n");
	msl::utl::printv("########################################\n");
	msl::utl::printv("# 4.2 testing communication skeletons  #\n");
	msl::utl::printv("########################################\n");
	
	msl::utl::printv("\nda = ");
	da1.show();
	msl::utl::printv("\nnote: allToAll not tested\n");
	msl::utl::printv("\nbroadcast,          da = ");
	da1.broadcast(3);
	da1.show();
	msl::utl::printv("\nbroadcastPartition, da = ");
	da2.broadcastPartition(1);
	da2.show();
	msl::utl::printv("\ngather,             da = [");
	da3.gather(b);

	for(int i = 0; i < SIZE; i++) {
		msl::utl::printv("%d", b[i]);

		if(i < SIZE - 1) {
			msl::utl::printv(", ");
		}
		else {
			msl::utl::printv("]\n");
		}
	}

	msl::utl::printv("permute,            da = ");
	da4.permute(rightShiftElement);
	da4.show();
	msl::utl::printv("\npermutePartition,   da = ");
	da5.permutePartition(leftShiftPartition);
	da5.show();

	msl::utl::printv("\n\nall communication skeleton tests passed!\n");
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::da::testDistributedArray();
	msl::TerminateSkeletons();
	return 0;
}

