// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include "Atomic.h"
#include "Curry.h"
#include "Farm.h"
#include "Final.h"
#include "Initial.h"
#include "Pipe.h"
#include "Muesli.h"
#include "TestFarm.h"
#include "Utility.h"

// test functions

void msl::test::farm::testFarm() {
	msl::utl::printv("Welcome to the test suite for the Farm skeleton.\n");

	if(Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");

		testConstructors();
		testDestructor();
		testFunctions();

		msl::utl::printv("\nall test passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

void msl::test::farm::testConstructors() {
	msl::Atomic<int, int> a(atomic, 1);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 1. testing constructors              *\n");
	msl::utl::printv("****************************************\n");

	msl::Farm<int, int> f(a, 1);
	
	msl::utl::printv("\nconstructor test passed!\n");
}

void msl::test::farm::testDestructor() {
	msl::Atomic<int, int> a(atomic, 1);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 2. testing destructor                *\n");
	msl::utl::printv("****************************************\n");

	msl::Farm<int, int>* f = new Farm<int, int>(a, 1);
	delete f;
	
	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::farm::testFunctions() {
	msl::Initial<int> i(initial);
	msl::Atomic<int, int> a(atomic, 1);
	msl::Farm<int, int> farm(a, 1);
	msl::Final<int> f(final);
	msl::Pipe p(i, farm, f);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 3. testing functions                 *\n");
	msl::utl::printv("****************************************\n");

	msl::utl::printv("\ntesting farm.copy()");
	msl::Farm<int, int>* copy = (msl::Farm<int, int>*)farm.copy();

	msl::utl::printv("\nnote: farm.setPredecessors() not tested");
	msl::utl::printv("\nnote: farm.setSuccessors() not tested");

	msl::utl::printv("\ntesting farm.show():\n");
	farm.show();

	msl::utl::printv("testing farm.start()");
	farm.start();
	
	msl::utl::printv("\n\nall function tests passed!\n");
}

// argument functions. note that these functions are
// only necessary to set up the skeleton topology.
// the constructed pipe is not intended to be started
// and therefore does not produce any outputs!

int* msl::test::farm::atomic(int* i) {
	int* result = new int;

	*result = (*i);

	(*result)++;

	return result;
}

int* msl::test::farm::initial(Empty e) {
	int* result = new int;

	if(counter == 10) {
		return NULL;
	}
	else {
		*result = counter++;
		return result;
	}
}

void msl::test::farm::final(int* i) {
	printf("%d\n", *i);
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::farm::testFarm();
	msl::TerminateSkeletons();
	return 0;
}
