// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include "Curry.h"
#include "Final.h"
#include "Muesli.h"
#include "TestFinal.h"
#include "Utility.h"

// main function

void msl::test::final::testFinal() {
	msl::utl::printv("Welcome to the test suite for the Final skeleton.\n");

	if(Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");

		testConstructors();
		testDestructor();
		testFunctions();

		msl::utl::printv("\nAll test have been successfully passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

// argument functions

void msl::test::final::f(int* i) {
	msl::utl::printv("i = %d\n", *i);
}

// test functions

void msl::test::final::testConstructors() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 1. testing constructors              *\n");
	msl::utl::printv("****************************************\n");

	msl::Final<int> final1(f);
	msl::Final<int> final2(DFct1<int*, void>(f));
	
	msl::utl::printv("\nall constructor tests passed!\n");
}

void msl::test::final::testDestructor() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 2. testing destructor                *\n");
	msl::utl::printv("****************************************\n");

	msl::Final<int>* final = new msl::Final<int>(f);
	delete final;
	
	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::final::testFunctions() {
	msl::Final<int> final(f);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 3. testing functions                 *\n");
	msl::utl::printv("****************************************\n");

	msl::utl::printv("\ntesting final.copy()");
	msl::Final<int>* copy = (msl::Final<int>*)final.copy();

	msl::utl::printv("\ntesting final.show(): ");
	final.show();

	msl::utl::printv("testing final.start()");
	final.start();
	
	msl::utl::printv("\n\nall function tests passed!\n");
}

int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::final::testFinal();
	msl::TerminateSkeletons();
	return 0;
}
