// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include "DistributedMatrix.h"
#include "Muesli.h"
#include "OAL.h"
#include "TestGauss.h"
#include "Utility.h"
#include <iostream>

double msl::test::gauss::init(const int a, const int b) {
	return (a == b) ? 1.0 : 2.0;
}

double msl::test::gauss::copyPivot(const msl::DistributedMatrix<double>& A,
									int k, int i, int j, double Pij) {
	return A.isLocal(k,k) ? A.getLocal(k, j, true) / A.getLocal(k, k, true) : 0;
}

double msl::test::gauss::pivotOp(const msl::DistributedMatrix<double>& A,
								  const msl::DistributedMatrix<double>& Pivot,
								  int k, int i, int j, double Pij) {
	double result = (i == k) ? Pivot.getLocal(i / A.getLocalRows(), j, true) : A.getLocal(i, j, true) -
		A.getLocal(i, k, true) * Pivot.getLocal(i / A.getLocalRows(), j, true);

	return result;
}

// idea:
// - each processor gets a sequence of rows of matrix A
// - repeatedly the pivot row is broadcasted to everybody
//   and each processor executes the pivot operation on
//   its partition of A
msl::DistributedMatrix<double> msl::test::gauss::gauss(const msl::DistributedMatrix<double>& A, int n) {
	// for simplicity assuming pivot != 0
	msl::DistributedMatrix<double> B(n, n + 1, 0.0, np, 1);
	msl::DistributedMatrix<double> ACopy(A);
	msl::DistributedMatrix<double> Pivot(np, n + 1, 0.0, np, 1);
	msl::DistributedMatrix<double>* Bptr = &B;
	msl::DistributedMatrix<double>* Aptr = &ACopy;
	msl::DistributedMatrix<double>* AuxPtr;
double time1 = 0; 
double time2 = 0; 
double time3 = 0;
double timetmp; 
	for(int k = 0; k < n; k++) {
	timetmp = MPI_Wtime();
		Pivot.mapIndexInPlace(curry(copyPivot)(*Aptr)(k));
	time1 += (MPI_Wtime() - timetmp);
	timetmp = MPI_Wtime();
		Pivot.broadcastPartition(k / (*Aptr).getLocalRows(), 0);
	time2 += (MPI_Wtime() - timetmp);
	timetmp = MPI_Wtime();
		(*Bptr).mapIndexInPlace(curry(pivotOp)(*Aptr)(Pivot)(k));
	time3 += (MPI_Wtime() - timetmp);
	timetmp = MPI_Wtime();
		AuxPtr = Bptr;
		Bptr = Aptr;
		Aptr = AuxPtr;
	}

std::cout << "Time Copy Pivot: " << time1 << "\nTime Broadcast: " << time2 << "\nTime Pivot Op: " << time3 << "\n";
	return *Aptr;
} 

// *************************************************************
// Test: Gaussian Elimination
// *************************************************************
void msl::test::gauss::testGauss(int n, int nt, bool flag) {
	//msl::utl::printv("Welcome to the test suite for the Gaussian Elimination.\n\n");
	np = Muesli::MSL_numOfTotalProcs;
	//msl::oal::setNumThreads(nt);
	DistributedMatrix<double> A(n, n + 1, &init, np, 1);

	if(flag) {
		msl::utl::printv("A =\n");
		A.show();
		msl::utl::printv("\n");
	}

	msl::DistributedMatrix<double> B = gauss(A, n);
	
	if(flag) {
		msl::utl::printv("B =\n");
		B.show();
		msl::utl::printv("\n");
	}
	
	//msl::utl::printv("All test have been successfully passed!\n\n");
}



int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	if (msl::Muesli::MSL_ARG1 > 0)
		msl::test::gauss::testGauss(msl::Muesli::MSL_ARG1, 12, false);
	else
		msl::test::gauss::testGauss(8, 12, true);
	msl::TerminateSkeletons();
	return 0;
}
