#include "DistributedMatrix.h"
#include "Muesli.h"
#include "OAL.h"
#include "Utility.h"
#include <iostream>

//#define Output 1

inline double init(const int a, const int b){return (a==b) ? 1.0 : 2.0;}

inline double copyPivot(const msl::DistributedMatrix<double>& A, int k, int i, int j, double Pij){
  return A.isLocal(k,k) ? A.getLocal(k,j,true)/A.getLocal(k,k,true) : 0;
}

struct pOp {
	int rows, firstrow, k, n;
	
	double operator() (double** A, double** Pivot) {
		#pragma omp parallel for
  		for (int l=0; l<rows; l++){
    		double Alk = A[l][k];
    		for (int j=k; j<=n; j++)
      			if (firstrow+l == k)
        			A[l][j] = Pivot[0][j];
        		else A[l][j] -=  Alk * Pivot[0][j];
  		}
	}
};

/*inline void pivotOp(const msl::DistributedMatrix<double>& Pivot, int rows, int firstrow,
            int k, int n, double** A){
  //double Alk;
  #pragma omp parallel for
  for (int l=0; l<rows; l++){
    double Alk = A[l][k];
    for (int j=k; j<=n; j++)
      if (firstrow+l == k)
        A[l][j] = Pivot.getLocalGlobal(0,j);
        else A[l][j] -=  1;//Alk * Pivot.getLocalGlobal(0,j);
  }
}*/

// idea: - each processor gets a sequence of rows of matrix A
//       - repeatedly the pivot row is broadcasted to everybody
//         and each processor executes the pivot operation on its partition of A
void gauss(msl::DistributedMatrix<double>& A, int n, int np){
	msl::DistributedMatrix<double> Pivot(np,n+1,0.0,np,1);
	pOp p;
	p.rows = A.getLocalRows();
	p.firstrow = A.getFirstRow();
	p.n = n;
  	for (int k=0; k<n; k++){
    	Pivot.mapIndexInPlace(curry(copyPivot)(A)(k));
    	Pivot.broadcastPartition(k/A.getLocalRows(),0);
    	//A.mapPartitionInPlace(curry(pivotOp)(Pivot,A.getLocalRows(),A.getFirstRow(),k, n));
		p.k = k;
		A.mapPartitionInPlace(p, Pivot.getPartition());
  	}
} 

int main(int argc, char **argv){
//*************************************************************
// Test: Gaussian Elimination
// *************************************************************
    msl::InitSkeletons(argc,argv);
   
	int n = msl::Muesli::MSL_ARG1;
	int np = msl::Muesli::MSL_numOfTotalProcs;

    msl::DistributedMatrix<double> A(n,n+1,&init,np,1);

//#if Output == 1
  //  if (msl::Muesli::MSL_myId == 0) {std::cout << "A:" << std::endl << std::flush;} 
    //A.show();
//#endif*/

    gauss(A, n, np);

//#if Output == 1
    //if (msl::Muesli::MSL_myId == 0) {std::cout << "Result of Gaussian Elimination:" << std::endl << std::flush;}
    //A.show();
//#endif

    msl::TerminateSkeletons();        
}
