// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include "Curry.h"
#include "Initial.h"
#include "Muesli.h"
#include "TestInitial.h"
#include "Utility.h"

// main function

void msl::test::initial::testInitial() {
	msl::utl::printv("Welcome to the test suite for the Initial skeleton.\n");

	if(Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");

		testConstructors();
		testDestructor();
		testFunctions();

		msl::utl::printv("\nAll test have been successfully passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

// argument functions

int* msl::test::initial::f(msl::Empty e) {
	int* result;

	if(I == 10) {
		*result = NULL;
	}
	else {
		*result = I++;
	}

	return result;
}

// test functions

void msl::test::initial::testConstructors() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 1. testing constructors              *\n");
	msl::utl::printv("****************************************\n");

	msl::Initial<int> init1(f);
	msl::Initial<int> init2(DFct1<int, msl::Empty>(f));
	
	msl::utl::printv("\nall constructor tests passed!\n");
}

void msl::test::initial::testDestructor() {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 2. testing destructor                *\n");
	msl::utl::printv("****************************************\n");

	msl::Initial<int>* init = new msl::Initial<int>(f);
	delete init;
	
	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::initial::testFunctions() {
	msl::Initial<int> init(f);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 3. testing functions                 *\n");
	msl::utl::printv("****************************************\n");

	msl::utl::printv("\ntesting init.copy()");
	msl::Initial<int>* copy = (msl::Initial<int>*)init.copy();

	msl::utl::printv("\ntesting init.show(): ");
	init.show();

	msl::utl::printv("testing init.start()");
	init.start();
	
	msl::utl::printv("\n\nall function tests passed!\n");
}

int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::initial::testInitial();
	msl::TerminateSkeletons();
	return 0;
}
