// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <iostream>
#include <math.h>

#include "DistributedMatrix.h"
#include "Muesli.h"
#include "OAL.h"
#include "TestMatMult.h"
#include "Utility.h"

int msl::test::matmult::negate(const int a) {
	return -a;
}

int msl::test::matmult::add(const int a, const int b) {
	return a + b;
}

template<class C> C msl::test::matmult::skprod(const msl::DistributedMatrix<C>& A,
												const msl::DistributedMatrix<C>& B,
												int i, int j, C Cij) {
	C sum = Cij;

	for(int k = 0; k < A.getLocalRows(); ++k)
		//sum += A.getGlobalLocal(i,k) * B.getLocalGlobal(k,j);
		sum += B.getLocalGlobal(k, j) * A.getGlobalLocal(i, k);

	return sum;
}

/* Using algorithm of Gentleman based on torus topology
*/
template<class C> 
msl::DistributedMatrix<C> msl::test::matmult::matmult(msl::DistributedMatrix<C> A,
																		 msl::DistributedMatrix<C> B) {
	// assumption: A, B have same square shape
	A.rotateRows(negate);
	// A.show();
	B.rotateCols(negate);
	// B.show();
	msl::DistributedMatrix<C> R(A.getRows(), A.getCols(), 0, A.getBlocksInCol(), A.getBlocksInRow());

double tTemp = 0;
double tRotate = 0;
double tComp = 0;

	for(int i = 0; i < A.getBlocksInRow(); ++i) {
		typedef C (*skprod_t)(const msl::DistributedMatrix<C>&,
			const msl::DistributedMatrix<C>&, int, int, C);
MPI_Barrier(MPI_COMM_WORLD);
tTemp = MPI_Wtime();
		R.mapIndexInPlace(curry((skprod_t)skprod<C>)(A)(B));
tComp += (MPI_Wtime() - tTemp); 
MPI_Barrier(MPI_COMM_WORLD);
tTemp = MPI_Wtime();
		A.rotateRows(-1);
		B.rotateCols(-1);
MPI_Barrier(MPI_COMM_WORLD);
tRotate += (MPI_Wtime() - tTemp);
	}

if (Muesli::MSL_myId == 0) std::cout << "rotate Time: " << tRotate << "\n";
	return R;
}

// *************************************************************
// Test: matrix multiplication
// *************************************************************

void msl::test::matmult::testMatMult(int n, bool flag) {
	// square root of np
	int sqrtp;
	// calculate square root of np
	sqrtp = (int)(sqrt((double)Muesli::MSL_numOfTotalProcs) + 0.1);
	// create matrix A
	msl::DistributedMatrix<int> A(n, n, &add, sqrtp, sqrtp);
	// create matrix B
	msl::DistributedMatrix<int> B(n, n, &add, sqrtp, sqrtp);
	
	// output matrix A
	if(flag) {
		msl::utl::printv("A =\n");
		//A.show();
		msl::utl::printv("\n");
	}
	
	// output matrix B
	if(flag) {
		msl::utl::printv("B =\n");
		//B.show();
		msl::utl::printv("\n");
	}

	msl::DistributedMatrix<int> C = matmult(A, B);

	// output matrix C
	if(flag) {
        msl::utl::printv("C (A x B) =\n");
		C.show();
	}

	//A.~DistributedMatrix();
	//B.~DistributedMatrix();
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	
	if (msl::Muesli::MSL_ARG1 > 0) {
		//warmup
		msl::test::matmult::testMatMult(msl::Muesli::MSL_ARG1,  false);

		for (int i = 0; i < 2; i++) 
			msl::test::matmult::testMatMult(msl::Muesli::MSL_ARG1,  false);
	}
	else
		msl::test::matmult::testMatMult(8, true);
	msl::TerminateSkeletons();
	return 0;
}
