// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <cstdarg>
#include <cstdio>

#include "Atomic.h"
#include "Curry.h"
#include "Final.h"
#include "Initial.h"
#include "Pipe.h"
#include "Muesli.h"
#include "TestPipe.h"
#include "Utility.h"


// main function

void msl::test::pipe::testPipe(bool flag) {
	msl::utl::printv("Welcome to the test suite for the Pipe skeleton.\n");

    if(flag) {
		msl::utl::printv("\nATTENTION: Although this function call\n");
		msl::utl::printv("should output the numbers from 1 to 10 it\n");
		msl::utl::printv("apparently doesn't. This is due to the\n");
		msl::utl::printv("fact that the Pipe skeleton obviously\n");
		msl::utl::printv("only works correctly, if the id of the\n");
		msl::utl::printv("first skeleton is 0. Otherwise, the first\n");
		msl::utl::printv("skeleton, in this case the Initial\n");
		msl::utl::printv("skeleton, aborts the execution of its\n");
		msl::utl::printv("start function, because the flag 'finished'\n");
		msl::utl::printv("is set to true. Everythings works fine\n");
		msl::utl::printv("though if you only create a single Pipe\n");
		msl::utl::printv("skeleton and call its start function.\n");
		msl::utl::printv("During this test suite, numerous Pipe\n");
		msl::utl::printv("skeletons have been created, such that\n");
		msl::utl::printv("this Initial skeleton is assigned the id 5.\n");
		msl::utl::printv("If you want to test the function and get a\n");
		msl::utl::printv("result, simply start this test suite by\n");
		msl::utl::printv("passing the argument 'false' to the main\n");
		msl::utl::printv("test function.\n");
	}

	if(Muesli::MSL_numOfTotalProcs == PROCESSORS) {
		msl::utl::printv("Note that only the first process prints its output!\n");

		if(flag) {
			testConstructors();
			testDestructor();
		}
		else {
			msl::utl::printv("Note: Only testing pipe.start()!\n");
		}

		testFunctions(flag);

		msl::utl::printv("\nAll tests have been succesfully passed!\n\n");
	}
	else {
		msl::utl::printv("Please restart this suite with np = %d processors!\n\n", PROCESSORS);
	}
}

// argument functions

int* msl::test::pipe::atomic(int* i) {
	int* result = new int;

	*result = (*i);

	(*result)++;

	return result;
}

int* msl::test::pipe::initial(msl::Empty e) {
	int* result = new int;

	if(I == 10) {
		return NULL;
	}
	else {
		*result = I++;
		return result;
	}
}

void msl::test::pipe::final(int* i) {
	printf("%d\n", *i);
}

// test functions

void msl::test::pipe::testConstructors() {
	msl::Atomic<int, int> a(atomic, 1);
	msl::Initial<int> i(initial);
	msl::Final<int> f(final);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 1. testing constructors              *\n");
	msl::utl::printv("****************************************\n");

	msl::utl::printv("\ntesting constructor with 2 arguments");
	msl::Pipe pipe1(i, f);
	msl::utl::printv("\ntesting constructor with 3 arguments");
	msl::Pipe pipe2(i, a, f);
	
	msl::utl::printv("\n\nall constructor tests passed!\n");
}

void msl::test::pipe::testDestructor() {
	msl::Initial<int> initial(initial);
	msl::Final<int> final(final);

	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 2. testing destructor                *\n");
	msl::utl::printv("****************************************\n");

	msl::Pipe* pipe = new msl::Pipe(initial, final);
	delete pipe;
	
	msl::utl::printv("\ndestructor test passed!\n");
}

void msl::test::pipe::testFunctions(bool flag) {
	msl::utl::printv("\n");
	msl::utl::printv("****************************************\n");
	msl::utl::printv("* 3. testing functions                 *\n");
	msl::utl::printv("****************************************\n");

	if(flag) {
		testCopy();
	}

	testStart(flag);
	
	msl::utl::printv("\nall function tests passed!\n");
}

void msl::test::pipe::testCopy() {
	msl::Initial<int> i(initial);
	msl::Atomic<int, int> a(atomic, 1);
	msl::Final<int> f(final);
	msl::Pipe p(i, a, f);

	msl::utl::printv("\ntesting pipe.copy()");
	msl::Pipe* copy = (msl::Pipe*)p.copy();

	msl::utl::printv("\nnote: pipe.setPredecessors() not tested");
	msl::utl::printv("\nnote: pipe.setSuccessors() not tested");

	msl::utl::printv("\ntesting pipe.show():\n");
	p.show();
}

void msl::test::pipe::testStart(bool flag) {
    msl::Initial<int> i(initial);
	msl::Atomic<int, int> a(atomic, 1);
	msl::Final<int> f(final);
	msl::Pipe p(i, a, f);
    
	msl::utl::printv("testing pipe.start()\n");
	p.start();

	
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::pipe::testPipe(false);
	msl::TerminateSkeletons();
	return 0;
}
