// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <stdlib.h>

#include "DistributedArray.h"
#include "Muesli.h"
#include "OAL.h"
#include "TestSamplesort.h"
#include "Utility.h"

int msl::test::samplesort::random(int i) {
	return rand() % 1000000;
}

template<class C> int* msl::test::samplesort::initIndex(const msl::DistributedArray<C>& A,
														 int selection[], int i) { 
	int* idx = new int[np + 1];
	idx[0] = 0;
	int j = 0;
	int k = 0;

	while((k <= A.getLocalSize()) && (j < np - 1))
		if(A.getLocal(k) >= selection[j])
			idx[++j] = k;
		else
			k++;

	for(k = j + 1; k <= np; k++)
		idx[k] = A.getLocalSize();

	return idx;
} 

template<class C> C msl::test::samplesort::takeSample(const msl::DistributedArray<C>& A, int i) {
	return A.get(i * Muesli::MSL_ARG1 / (np * np));
}

template<class C> C msl::test::samplesort::moveResult(C *sorted, int i, C x) {
	return sorted[i];
}

template<class C> void msl::test::samplesort::quicksort(int low, int high, C* _a) {
	typedef C* MutableA;
	MutableA a = const_cast<MutableA>(_a);

	if(low >= high)
		return;

	int left = low;
	int right = high;
	int pivot = a[(low + high) / 2];
	
	do {
		while(a[left] < pivot)
			left++;
		
		while(a[right] > pivot)
			right--;
		
		if(left <= right) {
			int temp = a[right];
			a[right--] = a[left];
			a[left++] = temp;
		}
	}
	while(left <= right);

	if(low < right)
		quicksort(low, right, a);

	if(high > left)
		quicksort(left, high, a);
}

template<class C> void msl::test::samplesort::samplesort(msl::DistributedArray<C>& A, C maxVal) {
	int i;
	int j = 0;
	int upperBound = 2 * Muesli::MSL_ARG1 / np; // could be refined
	typedef void (*quicksort_t)(int, int, C*);
	typedef C (*takeSample_t)(const msl::DistributedArray<C>&, int i);
	typedef int* (*initIndex_t)(const msl::DistributedArray<C>&, int*, int);
	typedef C (*moveResult_t)(C*, int i, C);

	A.mapPartitionInPlace(curry((quicksort_t)quicksort<C>)(0)(A.getLocalSize() - 1));
	msl::DistributedArray<C> Sample(np * np, curry((takeSample_t)takeSample<C>)(A));
	C* sample = new C[np * np];
	Sample.gather(sample);
	quicksort(0, np * np - 1, sample);
	C* selection = new C[np - 1];

	for(i = 0; i < np - 1; i++)
		selection[i] = sample[(i + 1) * np + np / 2 - 1];
	
	msl::DistributedArray<int*> Index(np, curry((initIndex_t)initIndex<C>)(A)(selection));
	msl::DistributedArray<C>* B = A.copyWithGap(np * upperBound, maxVal);
	B->allToAll(Index, maxVal); // gaps are filled with maxVal
	B->mapPartitionInPlace(curry((quicksort_t)quicksort<C>)(0)(upperBound - 1));
	C* sorted = new C[np * upperBound];
	B->gather(sorted);

	for(i = 0; i < np * upperBound; i++) // compaction (removes gaps)
		if(sorted[i] != maxVal)
			sorted[j++] = sorted[i];

	A.mapIndexInPlace(curry((moveResult_t)moveResult<C>)(sorted));
	
	delete[] sample;
	delete[] selection;
	delete[] sorted;
}

//*****************************************************************************
// Test: samplesort (algorithm from M. Quinn: Parallel Computing, 1994, p. 281)
// ****************************************************************************
void msl::test::samplesort::testSampleSort(int n, int nt, bool flag) {
	Muesli::MSL_ARG1 = n;
	np = Muesli::MSL_numOfTotalProcs;

	srand(Muesli::MSL_myId);
	msl::oal::setNumThreads(nt);

	msl::DistributedArray<int> A(n, &random);

	if(flag) {
		msl::utl::printv("A (unsorted): ");
		A.show();
		msl::utl::printv("\n");
	}

	samplesort(A, 9999);
	
	if(flag) {
		msl::utl::printv("A (sorted):   ");
		A.show();
		msl::utl::printv("\n\n");
	}
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	msl::test::samplesort::testSampleSort(8, 1, true);
	msl::TerminateSkeletons();
	return 0;
}
