// **********************************************************************************************************
// *																										*
// * This Software is published under the MIT License.														*
// * For more information and the license text, please refer to the file 'License' located in the zip file. *
// *																										*
// * Copyright (c) 2001-2011 Steffen Ernsting, Philipp Ciechanowicz, Michael Poldner and Herbert Kuchen     *                 
// *        		{s.ernsting|ciechanowicz|poldner|kuchen}@uni-muenster.de              					*
// *																										*	
// **********************************************************************************************************

#include <iostream>

#include "TestVarious.h"
#include "Muesli.h"
#include "DistributedMatrix.h"
#include "DistributedArray.h"
#include "Utility.h"

int msl::test::various::add(const int a, const int b) {
	return a + b;
}

int msl::test::various::init(const int a) {
	return a;
}

int permute(const int id) {
	return (id + 1) % 4;
}

int newRow(const int row, const int col) {
	return (row + 1) % 2;
}

int newCol(const int row, const int col) {
	return (col + 1) % 2;
}


void msl::test::various::test(int n) {
	int sqrtp = (int)(sqrt((double)Muesli::MSL_numOfTotalProcs) + 0.1);
	msl::DistributedMatrix<int> A(n, n, &add, sqrtp, sqrtp);
	msl::DistributedArray<int> B(n, &init);
	
	//A.show();
	msl::utl::printv("permuting A...\n");
	A.permutePartition(&newRow, &newCol);
	//A.show();
	msl::utl::printv("broadcasting Partition (0, 0)... \n");
	A.broadcastPartition(0, 0);
	//A.show();
	
	msl::utl::printv("\n");
	//B.show();
	msl::utl::printv("permuting B...\n");	
	B.permutePartition(&permute);
	//B.show();
}


int main(int argc, char** argv) {
	msl::InitSkeletons(argc, argv);
	if (msl::Muesli::MSL_ARG1 > 0)
		msl::test::various::test(msl::Muesli::MSL_ARG1);
	else
		msl::test::various::test(4);
	msl::TerminateSkeletons();
	return 0;
}

